#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#define NO_LEDS 10

int *my_ddrs[NO_LEDS];
int *my_ports[NO_LEDS];
int my_leds[NO_LEDS] = {PD1, PD0, PB7, PB6, PB5, PB4, PB3, PB2, PB1, PB0};

int alcohol_value = 0;

void init_devices() {
    int i;

    for (i = 2; i < NO_LEDS; ++i) {
        my_ddrs[i] = (int*)&DDRB;
        my_ports[i] = (int*)&PORTB;
    }

    for (i = 0; i < 2; ++i) {
        my_ddrs[i] = (int*)&DDRD;
        my_ports[i] = (int*)&PORTD;
    }

    /* setez butonul ca intrare */
	DDRD &= ~(1 << PD6);
	/* activez rezistenta de pull-up pentru buton */
	PORTD |= (1 << PD6);

    /* setez ledurile ca iesiri si ma asigur ca sunt stinse */
    for (i = 0; i < NO_LEDS; ++i) {
        *(my_ddrs[i]) |= (1 << my_leds[i]);
        *(my_ports[i]) &= ~(1 << my_leds[i]);
    }

    /* seteaza ledul de langa pwr ca iesire */
    DDRD |= (1 << PD7);

}

void turn_on_leds(int leds_to_turn_on) {
    int i;
    for (i = 0; i < leds_to_turn_on; ++i) {
        *(my_ports[i]) |= (1 << my_leds[i]);
    }

    for (i = leds_to_turn_on; i < NO_LEDS; i++) {
        *(my_ports[i]) &= ~(1 << my_leds[i]);
    }
}

void turn_off_leds() {
    int i;
    for (i = 0; i < NO_LEDS; ++i) {
        *(my_ports[i]) &= ~(1 << my_leds[i]);
    }
}

void ADC_init(void) {
    // enable ADC with:
    // *reference AVCC with external capacitor at AREF pin
    // *without left adjust of conversion result
    // *no auto-trigger
    // *no interrupt
    // *prescaler at 128
    ADMUX = (1 << REFS0);
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

// read adc value
uint16_t adc_read(uint8_t ch) {
    // select the corresponding channel 0~7
    // ANDing with '7' will always keep the value
    // of 'ch' between 0 and 7
    ch &= 0b00000111;  // AND operation with 7
    ADMUX = (ADMUX & 0xF8)|ch;     // clears the bottom 3 bits before ORing

    // start single conversion
    // write '1' to ADSC
    ADCSRA |= (1<<ADSC);

    // wait for conversion to complete
    // ADSC becomes '0' again
    // till then, run loop continuously
    while(ADCSRA & (1<<ADSC));

    return (ADC);
}

/* Perform a linear mapping from [500, 1000] to [0, 10] */
int get_safe_value(int alcohol_value) {

    if (alcohol_value < 500) {
        return 0;
    }

    if (alcohol_value > 1000) {
        return NO_LEDS;
    }

    int safe_value = 0.02 * (alcohol_value - 500);

    return safe_value;
}

int leds_to_turn_on = 0;
/* aprinde led-ul PC0 la apasarea butonului PB0 */
void detect_alcohol() {
    while (1) {
        /* daca butonul este apasat */
        if ((PIND & (1 << PD6)) == 0) {
            /* ledul este aprins */
            alcohol_value = adc_read(3);
            
            PORTD |= (1 << PD7);
            leds_to_turn_on = get_safe_value(alcohol_value);

            turn_on_leds(leds_to_turn_on);
        } else {
            /* altfel ledul este stins */
            turn_off_leds();
            PORTD &= ~(1 << PD7);
        }
    }
}

int main(void) {
    init_devices();
    ADC_init();
    detect_alcohol();

    return 0;
}

